#include "HashPool.h"

bool IsNum ( const char* sz )
{
	const char szSet [ 15 ] = "1234567890";
	if ( strspn( sz, szSet ) == strlen( sz ) )
	{
		return true;
	}
	return false;
}

CHashPool::CHashPool( const char* sz )
{
	strncpy( m_szName, sz, 128 );
	strcpy( m_szFile, "" );
}

CHashPool::~CHashPool(void)
{
	m_pHashMap.clear();
}

void CHashPool::SetFileName( const char* sz )
{
	if ( ( strlen( sz ) < 12 ) || ( ( sz[ 0 ] != 's' ) && ( sz[ 1 ] != 'c' ) && ( sz[ 2 ] != 'r' ) && ( sz[ 3 ] != 'i' ) && ( sz[ 4 ] != 'p' ) && ( sz[ 5 ] != 't' ) && ( sz[ 6 ] != 'f' ) && ( sz[ 7 ] != 'i' ) && ( sz[ 8 ] != 'l' ) && ( sz[ 9 ] != 'e' ) && ( sz[ 10 ] != 's' ) ) )
	{
		strcpy( m_szFile, "scriptfiles/" );
		strncat( m_szFile, sz, MAX_PATH );
	}
	else strncpy( m_szFile, sz, MAX_PATH );
}

bool CHashPool::Add( const char *szSection, const char *szData )
{
	m_pHashMap[ std::string( szSection ) ] = std::string( szData );
	return true;
}

const char* CHashPool::Get( const char* szSection )
{
	return m_pHashMap[ std::string( szSection ) ].c_str();
}

const char* CHashPool::GetIndex( unsigned int uiIndex )
{
	if ( uiIndex < m_pHashMap.bucket_count() )
	{
		stdext::hash_map< std::string, std::string >::const_iterator iter = m_pHashMap.begin();
		for ( unsigned int ui = 0; ui < uiIndex; ui++ )
		{
			iter++;
		}

		return (*iter).second.c_str();
	}

	return "";
}

bool CHashPool::Inc( const char *szSection, unsigned int uiAmmount )
{
	if ( m_pHashMap[ szSection ].length() == 0 ) 
	{
		char szData[ 128 ] = { 0 };
		strncpy( szData, m_pHashMap[ szSection ].c_str(), 128 );
		if ( IsNum( szData ) )
		{
			int ui = static_cast< int >( atoi( szData ) );
			ui += uiAmmount;
			sprintf( szData, "%i", ui );

			m_pHashMap[ szSection ] = szData;

			return true;
		}
	}
	else
	{
		char szData[ 128 ] = { 0 };
		sprintf( szData, "%i", uiAmmount );

		m_pHashMap[ szSection ] = szData;

		return true;
	}
	return false;
}

bool CHashPool::Dec( const char *szSection, int uiAmmount )
{
	if ( m_pHashMap[ szSection ].length() == 0 ) 
	{
		char szData[ 128 ] = { 0 };
		strncpy( szData, m_pHashMap[ szSection ].c_str(), 128 );
		if ( IsNum( szData ) )
		{
			int ui = static_cast< int >( atoi( szData ) );
			ui -= uiAmmount;
			sprintf( szData, "%i", ui );

			m_pHashMap[ szSection ] = szData;

			return true;
		}
	}
	else
	{
		char szData[ 128 ] = { 0 };
		sprintf( szData, "%i", -uiAmmount );

		m_pHashMap[ szSection ] = szData;

		return true;
	}

	return false;
}

bool CHashPool::Del( const char *szSection )
{
	m_pHashMap.erase( szSection );
	return true;
}

bool CHashPool::Load( void )
{
	if ( !*m_szFile ) return false;

	FILE* pFile = fopen( m_szFile, "r" );

	if ( pFile )
	{
		m_pHashMap.clear();

		char szLine[ 128 ] = { 0 };

		while ( fgets( szLine, 128, pFile ) )
		{
			if ( ( *szLine ) && ( szLine[ 0 ] != '\n' ) )
			{
				char szSection[ 128 ] = { 0 };
				strncpy( szSection, szLine, 128 );
				szSection[ strlen( szSection ) - 1 ] = '\0';
				fgets( szLine, 128, pFile ); // We fgets again to get the new line
				szLine[ strlen( szLine ) - 1 ] = '\0';
				m_pHashMap[ szSection ] = szLine;
			}
		}

		fclose( pFile );

		return true;
	}

	return false;
}

bool CHashPool::Save( void )
{
	if ( !*m_szFile ) return false;

	FILE* pFile = fopen( m_szFile, "w+" );

	if ( pFile )
	{
		unsigned int ui = 0;
		stdext::hash_map< std::string, std::string >::const_iterator iter = m_pHashMap.begin();
		stdext::hash_map< std::string, std::string >::const_iterator iter_end = m_pHashMap.end();
		char szSection[ 128 + 1 ] = { 0 };
		char szData[ 128 + 1 ] = { 0 };
		for(; iter != iter_end; iter++ )
		{
			char szLineBreak[ 4 ] = { "\n" };
			strncpy( szSection, (*iter).first.c_str(), 128 );
			strncpy( szData, (*iter).second.c_str(), 128 );
			if ( ( *szSection ) && ( *szData ) )
			{
				fwrite( szSection, 1, strlen( szSection ), pFile );
				fwrite( szLineBreak, 1, strlen( szLineBreak ), pFile );
				fwrite( szData, 1, strlen( szData ), pFile );
				fwrite( szLineBreak, 1, strlen( szLineBreak ), pFile );
			}
		}

		fclose( pFile );

		return true;
	}

	return false;
}

// The Manager
unsigned int CHashPoolManager::m_uiHashes = 0;
stdext::hash_map< std::string, CHashPool* > CHashPoolManager::m_pHashes;

CHashPool* CHashPoolManager::New( const char* sz )
{
	CHashPool* pHash = new CHashPool( sz );
	if ( pHash )
	{
		m_pHashes[ sz ] = pHash;
		m_uiHashes++;

		return pHash;
	}

	return 0;
}

CHashPool* CHashPoolManager::Find( const char* sz )
{
	return m_pHashes[ sz ];
}

bool CHashPoolManager::Remove( CHashPool* p )
{
	if ( p )
	{
		m_pHashes[ p->GetName() ] = 0;
		m_uiHashes--;
		delete p;

		return true;
	}
	return false;
}

bool CHashPoolManager::Remove( const char* sz )
{
	CHashPool* p = Find( sz );
	if ( p )
	{
		m_pHashes[ sz ] = 0;
		m_uiHashes--;
		delete p;

		return true;
	}
	return false;
}

void CHashPoolManager::RemoveAll( void )
{
	stdext::hash_map< std::string, CHashPool* >::const_iterator iter = m_pHashes.begin();
	stdext::hash_map< std::string, CHashPool* >::const_iterator iter_end = m_pHashes.end();
	
	for( ; iter != iter_end; iter++ )
	{
		delete (*iter).second;
	}

	m_uiHashes = 0;
	m_pHashes.clear();
}