#pragma once

#include "main.h"
#ifdef _WIN32
#include <hash_map>
#else
#include <ext/hash_map>
namespace stdext = __gnu_cxx;

namespace __gnu_cxx
{ 
template<> struct hash< std::string > 
{ 
size_t operator()( const std::string& x ) const 
{ 
return hash< const char* >()( x.c_str() ); 
} 
}; 
} 
#endif

class CHashPool
{
public:
	CHashPool( const char* sz );
	~CHashPool(void);

	const char*							GetName				( void ) const				{ return m_szName; }
	void								SetName				( const char* sz )			{ strncpy( m_szName, sz, 128 ); }

	const char*							GetFileName			( void ) const				{ return m_szFile; }
	void								SetFileName			( const char* sz );

	bool								Add					( const char* szSection, const char* szData );
	bool								Inc					( const char* szSection, unsigned int uiAmmount = 1 );
	bool								Dec					( const char* szSection, int uiAmmount = 1 );
	bool								Del					( const char* szSection );

	const char*							Get					( const char* szSection );

	const char*							GetIndex			( unsigned int uiIndex );

	bool								Load				( void );
	bool								Save				( void );
	bool								Delete				( void );

private:
	char								m_szName[ 128 ];
	char								m_szFile[ MAX_PATH ];

	stdext::hash_map< std::string, std::string > m_pHashMap;
};

class CHashPoolManager
{
public:
	static CHashPool*					New					( const char* sz );

	static CHashPool*					Find				( const char* sz );

	static bool							Remove				( CHashPool* p );
	static bool							Remove				( const char* sz );

	static void							RemoveAll			( void );

	static unsigned int					Count				( void )					{ return m_uiHashes; }

private:
	static unsigned int					m_uiHashes;
	static stdext::hash_map< std::string, CHashPool* > m_pHashes;
};
