//----------------------------------------------------------
//
//   SA:MP Multiplayer Modification For GTA:SA
//   Copyright 2004-2007 SA:MP Team
//
//----------------------------------------------------------

#include "main.h"
#include "HashPool.h"
#include "CAmxString.h"

//----------------------------------------------------------

logprintf_t logprintf;
void **ppPluginData;
extern void *pAMXFunctions;

//----------------------------------------------------------
// The Support() function indicates what possibilities this
// plugin has. The SUPPORTS_VERSION flag is required to check
// for compatibility with the server. 

PLUGIN_EXPORT unsigned int PLUGIN_CALL Supports() 
{
	return SUPPORTS_VERSION | SUPPORTS_AMX_NATIVES;
}

//----------------------------------------------------------
// The Load() function gets passed on exported functions from
// the SA-MP Server, like the AMX Functions and logprintf().
// Should return true if loading the plugin has succeeded.

PLUGIN_EXPORT bool PLUGIN_CALL Load( void **ppData ) 
{
	pAMXFunctions = ppData[PLUGIN_DATA_AMX_EXPORTS];
	logprintf = (logprintf_t)ppData[PLUGIN_DATA_LOGPRINTF];

	logprintf( "Plugin hashloader.dll got loaded." );
	return true;
}

//----------------------------------------------------------
// The Unload() function is called when the server shuts down,
// meaning this plugin gets shut down with it.

PLUGIN_EXPORT void PLUGIN_CALL Unload( )
{
	CHashPoolManager::RemoveAll();
	logprintf( "Plugin hashloader.dll got unloaded." );
}

//----------------------------------------------------------
// This is the sourcecode of the HelloWorld pawn native that we
// will be adding. "amx" is a pointer to the AMX-instance that's
// calling the function (e.g. a filterscript) and "params" is
// an array to the passed parameters. The first entry (params[0])
// is equal to  4 * PassedParameters, e.g. 16 for 4 parameters.

// native hLoad();
static cell AMX_NATIVE_CALL n_hLoad( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );
	if ( !pHash ) pHash = CHashPoolManager::New( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		pHash->SetName( StrAmx::GetString(amx, params[ 1 ] ).c_str() );
		pHash->SetFileName( StrAmx::GetString(amx, params[ 2 ] ).c_str() );

		logprintf( "Loading hash %s from %s", StrAmx::GetString(amx, params[ 1 ] ).c_str(), pHash->GetFileName() );

		pHash->Load();

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hSave( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		logprintf( "Saving hash %s to %s", StrAmx::GetString(amx, params[ 1 ] ).c_str(), pHash->GetFileName() );
		pHash->Save();

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hClose( AMX* amx, cell* params )
{
	if ( CHashPoolManager::Remove( StrAmx::GetString(amx, params[ 1 ] ).c_str() ) ) return 1;

	return 0;
}

static cell AMX_NATIVE_CALL n_hGet( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		StrAmx::SetString( amx, params[ 3 ], pHash->Get( StrAmx::GetString(amx, params[ 2 ] ).c_str() ) );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hGetInt( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		return atoi( pHash->Get( StrAmx::GetString(amx, params[ 2 ] ).c_str() ) );
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hGetFloat( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	cell *cCPointer;
	amx_GetAddr(amx, params[3],&cCPointer);

	if ( pHash )
	{
		float f = static_cast< float >( atof( pHash->Get( StrAmx::GetString(amx, params[ 2 ] ).c_str() ) ) );
		*cCPointer = amx_ftoc( f );
		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hGetIndex( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		StrAmx::SetString( amx, params[ 3 ], pHash->GetIndex(  params[ 2 ] ) );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hAdd( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		pHash->Add( StrAmx::GetString(amx, params[ 2 ] ).c_str(), StrAmx::GetString(amx, params[ 3 ] ).c_str() );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hAddInt( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		char szData[ 128 ] = { 0 };
		sprintf( szData, "%i", params[ 3 ] );
		pHash->Add( StrAmx::GetString(amx, params[ 2 ] ).c_str(), szData );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hAddFloat( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		char szData[ 128 ] = { 0 };
		sprintf( szData, "%f", amx_ctof( params[ 3 ] ) );
		pHash->Add( StrAmx::GetString(amx, params[ 2 ] ).c_str(), szData );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hInc( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		if ( ( params[ 0 ] / 4 ) == 3 ) pHash->Inc( StrAmx::GetString(amx, params[ 2 ] ).c_str(), params[ 3 ] );
		pHash->Inc( StrAmx::GetString(amx, params[ 2 ] ).c_str() );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hDec( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		if ( ( params[ 0 ] / 4 ) == 3 ) pHash->Inc( StrAmx::GetString(amx, params[ 2 ] ).c_str(), params[ 3 ] );
		pHash->Inc( StrAmx::GetString(amx, params[ 2 ] ).c_str() );

		return 1;
	}
	return 0;
}

static cell AMX_NATIVE_CALL n_hDel( AMX* amx, cell* params )
{
	CHashPool* pHash = CHashPoolManager::Find( StrAmx::GetString(amx, params[ 1 ] ).c_str() );

	if ( pHash )
	{
		pHash->Del( StrAmx::GetString(amx, params[ 2 ] ).c_str() );

		return 1;
	}
	return 0;
}

// And an array containing the native function-names and the functions
// specified with them, e.g. HLoad -> n_hLoad in this example.
AMX_NATIVE_INFO HashLoaderNatives[ ] =
{
	{ "HLoad",				n_hLoad },
	{ "HSave",				n_hSave },
	{ "HClose",				n_hClose },
	{ "HAdd",				n_hAdd },
	{ "HAddInt",			n_hAddInt },
	{ "HAddFloat",			n_hAddFloat },
	{ "HGet",				n_hGet },
	{ "HGetInt",			n_hGetInt },
	{ "HGetFloat",			n_hGetFloat },
	{ "HGetIndex",			n_hGetIndex },
	{ "HInc",				n_hInc },
	{ "HDec",				n_hDec },
	{ "HDel",				n_hDel },
	{ 0,					0 }
};

//----------------------------------------------------------
// The AmxLoad() function gets called when a new gamemode or
// filterscript gets loaded with the server. In here we register
// the native functions we like to add to the scripts.

PLUGIN_EXPORT int PLUGIN_CALL AmxLoad( AMX *amx ) 
{
	return amx_Register( amx, HashLoaderNatives, -1 );
}

//----------------------------------------------------------
// When a gamemode is over or a filterscript gets unloaded, this
// function gets called. No special actions needed in here.

PLUGIN_EXPORT int PLUGIN_CALL AmxUnload( AMX *amx ) 
{
	return AMX_ERR_NONE;
}
